@@ -60,6 +60,13 @@ service_opts = [
                       'SSL is used.')),
     cfg.StrOpt('region_name_for_services',
                help=_('Default region name used to get services endpoints.')),
+    cfg.StrOpt('region_name_for_shared_services',
+               help=_('Region name for shared services endpoints.')),
+    cfg.ListOpt('shared_services_types',
+                default=['image', 'volume', 'volumev2'],
+                help=_('The shared services located in the other region.'
+                       'Needs region_name_for_shared_services option to '
+                       'be set for this to take effect.')),
     cfg.StrOpt('heat_stack_user_role',
                default="heat_stack_user",
                help=_('Keystone role for heat template-defined users.')),
@@ -124,6 +131,18 @@ engine_opts = [
                help=_('Allow reauthentication on token expiry, such that'
                       ' long-running tasks may complete. Note this defeats'
                       ' the expiry of any provided user tokens.')),
+    cfg.BoolOpt('allow_trusts_redelegation',
+                default=False,
+                help=_('Create trusts with redelegation enabled. '
+                       'This option is only used when '
+                       'reauthentication_auth_method is set to "trusts". '
+                       'Note that enabling this option does have '
+                       'security implications as all trusts created by Heat '
+                       'will use both impersonation and redelegation enabled. '
+                       'Enable it only when there are other services that '
+                       'need to create trusts from tokens Heat uses to '
+                       'access them, examples are Aodh and Heat in another '
+                       'region when configured to use trusts too.')),
     cfg.ListOpt('trusts_delegated_roles',
                 default=[],
                 help=_('Subset of trustor roles to be delegated to heat.'
@@ -164,6 +183,11 @@ engine_opts = [
                         'this limitation, any nova feature supported with '
                         'microversion number above max_nova_api_microversion '
                         'will not be available.')),
+    cfg.FloatOpt('max_ironic_api_microversion',
+                 help=_('Maximum ironic API version for client plugin. With '
+                        'this limitation, any ironic feature supported with '
+                        'microversion number above '
+                        'max_ironic_api_microversion will not be available.')),
     cfg.IntOpt('event_purge_batch_size',
                min=1,
                default=200,
@@ -436,7 +460,7 @@ def list_opts():
     for client in ('aodh', 'barbican', 'cinder', 'designate',
                    'glance', 'heat', 'keystone', 'magnum', 'manila', 'mistral',
                    'monasca', 'neutron', 'nova', 'octavia', 'sahara', 'senlin',
-                   'swift', 'trove', 'zaqar'
+                   'swift', 'trove', 'vitrage', 'zaqar'
                    ):
         client_specific_group = 'clients_' + client
         yield client_specific_group, clients_opts
