@@ -11,8 +11,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import functools
+
 from keystoneauth1 import access
-from keystoneauth1 import exceptions as ksa_exceptions
 from keystoneauth1.identity import access as access_plugin
 from keystoneauth1.identity import generic
 from keystoneauth1 import loading as ks_loading
@@ -24,8 +25,6 @@ from oslo_log import log as logging
 import oslo_messaging
 from oslo_middleware import request_id as oslo_request_id
 from oslo_utils import importutils
-import six
-import tenacity
 
 from heat.common import config
 from heat.common import endpoint_utils
@@ -38,6 +37,8 @@ from heat.engine import clients
 LOG = logging.getLogger(__name__)
 
 
+cfg.CONF.import_opt('client_retry_limit', 'heat.common.config')
+
 # Note, we yield the options via list_opts to enable generation of the
 # sample heat.conf, but we don't register these options directly via
 # cfg.CONF.register*, it's done via ks_loading.register_auth_conf_options
@@ -53,15 +54,6 @@ TRUSTEE_CONF_GROUP = 'trustee'
 ks_loading.register_auth_conf_options(cfg.CONF, TRUSTEE_CONF_GROUP)
 
 
-retry_on_connection_timeout = tenacity.retry(
-    stop=tenacity.stop_after_attempt(cfg.CONF.client_retry_limit+1),
-    wait=tenacity.wait_random(max=2),
-    retry=tenacity.retry_if_exception_type(
-        (ksa_exceptions.ConnectFailure,
-         ksa_exceptions.DiscoveryFailure)),
-    reraise=True)
-
-
 def list_opts():
     trustee_opts = ks_loading.get_auth_common_conf_options()
     trustee_opts.extend(ks_loading.get_auth_plugin_conf_options(
@@ -122,6 +114,7 @@ class RequestContext(context.RequestContext):
         self._session = None
         self._clients = None
         self._keystone_session = session.Session(
+            connect_retries=cfg.CONF.client_retry_limit,
             **config.get_ssl_options('keystone'))
         self.trust_id = trust_id
         self.trustor_user_id = trustor_user_id
@@ -299,8 +292,6 @@ class RequestContext(context.RequestContext):
 
 
 class StoredContext(RequestContext):
-
-    @retry_on_connection_timeout
     def _load_keystone_data(self):
         self._keystone_loaded = True
         auth_ref = self.auth_plugin.get_access(self.keystone_session)
@@ -417,7 +408,7 @@ def ContextMiddleware_filter_factory(global_conf, **local_conf):
 
 
 def request_context(func):
-    @six.wraps(func)
+    @functools.wraps(func)
     def wrapped(self, ctx, *args, **kwargs):
         try:
             return func(self, ctx, *args, **kwargs)
