@@ -13,6 +13,7 @@
 
 """Implementation of SQLAlchemy backend."""
 import datetime
+import functools
 import itertools
 import random
 
@@ -24,9 +25,9 @@ from oslo_db.sqlalchemy import enginefacade
 from oslo_db.sqlalchemy import utils
 from oslo_log import log as logging
 from oslo_utils import encodeutils
+from oslo_utils import excutils
 from oslo_utils import timeutils
 import osprofiler.sqlalchemy
-import six
 import sqlalchemy
 from sqlalchemy import and_
 from sqlalchemy import func
@@ -88,9 +89,31 @@ def get_session():
     return get_facade().get_session()
 
 
+def retry_on_db_error(func):
+    @functools.wraps(func)
+    def try_func(context, *args, **kwargs):
+        if (context.session.transaction is None or
+                not context.session.autocommit):
+            wrapped = oslo_db_api.wrap_db_retry(max_retries=3,
+                                                retry_on_deadlock=True,
+                                                retry_on_disconnect=True,
+                                                retry_interval=0.5,
+                                                inc_retry_interval=True)(func)
+            return wrapped(context, *args, **kwargs)
+        else:
+            try:
+                return func(context, *args, **kwargs)
+            except (db_exception.DBDeadlock, db_exception.DBConnectionError):
+                with excutils.save_and_reraise_exception():
+                    LOG.debug('Not retrying on DBDeadlock and '
+                              'DBConnectionError because '
+                              'transaction not closed')
+    return try_func
+
+
 def update_and_save(context, obj, values):
     with context.session.begin(subtransactions=True):
-        for k, v in six.iteritems(values):
+        for k, v in values.items():
             setattr(obj, k, v)
 
 
@@ -142,7 +165,11 @@ def raw_template_update(context, template_id, values):
 
 
 def raw_template_delete(context, template_id):
-    raw_template = raw_template_get(context, template_id)
+    try:
+        raw_template = raw_template_get(context, template_id)
+    except exception.NotFound:
+        # Ignore not found
+        return
     raw_tmpl_files_id = raw_template.files_id
     session = context.session
     with session.begin(subtransactions=True):
@@ -153,7 +180,12 @@ def raw_template_delete(context, template_id):
         # delete that too
         if session.query(models.RawTemplate).filter_by(
                 files_id=raw_tmpl_files_id).first() is None:
-            raw_tmpl_files = raw_template_files_get(context, raw_tmpl_files_id)
+            try:
+                raw_tmpl_files = raw_template_files_get(
+                    context, raw_tmpl_files_id)
+            except exception.NotFound:
+                # Ignore not found
+                return
             session.delete(raw_tmpl_files)
 
 
@@ -234,14 +266,13 @@ def resource_get_all(context):
     return results
 
 
-@oslo_db_api.wrap_db_retry(max_retries=3, retry_on_deadlock=True,
-                           retry_interval=0.5, inc_retry_interval=True)
+@retry_on_db_error
 def resource_purge_deleted(context, stack_id):
     filters = {'stack_id': stack_id, 'action': 'DELETE', 'status': 'COMPLETE'}
     query = context.session.query(models.Resource)
     result = query.filter_by(**filters)
     attr_ids = [r.attr_data_id for r in result if r.attr_data_id is not None]
-    with context.session.begin(subtransactions=True):
+    with context.session.begin():
         result.delete()
         if attr_ids:
             context.session.query(models.ResourcePropertiesData).filter(
@@ -256,8 +287,7 @@ def _add_atomic_key_to_values(values, atomic_key):
         values['atomic_key'] = atomic_key + 1
 
 
-@oslo_db_api.wrap_db_retry(max_retries=3, retry_on_deadlock=True,
-                           retry_interval=0.5, inc_retry_interval=True)
+@retry_on_db_error
 def resource_update(context, resource_id, values, atomic_key,
                     expected_engine_id=None):
     return _try_resource_update(context, resource_id, values, atomic_key,
@@ -295,7 +325,7 @@ def resource_delete(context, resource_id):
 
 def resource_attr_id_set(context, resource_id, atomic_key, attr_id):
     session = context.session
-    with session.begin(subtransactions=True):
+    with session.begin():
         values = {'attr_data_id': attr_id}
         _add_atomic_key_to_values(values, atomic_key)
         rows_updated = session.query(models.Resource).filter(and_(
@@ -321,7 +351,7 @@ def resource_attr_id_set(context, resource_id, atomic_key, attr_id):
 
 def resource_attr_data_delete(context, resource_id, attr_id):
     session = context.session
-    with session.begin(subtransactions=True):
+    with session.begin():
         resource = session.query(models.Resource).get(resource_id)
         attr_prop_data = session.query(
             models.ResourcePropertiesData).get(attr_id)
@@ -443,7 +473,7 @@ def resource_exchange_stacks(context, resource_id1, resource_id2):
 def resource_data_delete(context, resource_id, key):
     result = resource_data_get_by_key(context, resource_id, key)
     session = context.session
-    with session.begin(subtransactions=True):
+    with session.begin():
         session.delete(result)
 
 
@@ -454,18 +484,16 @@ def resource_create(context, values):
     return resource_ref
 
 
-@oslo_db_api.wrap_db_retry(max_retries=3, retry_on_deadlock=True,
-                           retry_interval=0.5, inc_retry_interval=True)
+@retry_on_db_error
 def resource_create_replacement(context,
-                                existing_res_id, existing_res_values,
+                                existing_res_id,
                                 new_res_values,
                                 atomic_key, expected_engine_id=None):
     session = context.session
     try:
-        with session.begin(subtransactions=True):
+        with session.begin():
             new_res = resource_create(context, new_res_values)
             update_data = {'replaced_by': new_res.id}
-            update_data.update(existing_res_values)
             if not _try_resource_update(context,
                                         existing_res_id, update_data,
                                         atomic_key,
@@ -579,7 +607,7 @@ def stack_get_by_name(context, stack_name):
              models.Stack.tenant == context.tenant_id,
              models.Stack.stack_user_project_id == context.tenant_id)
              ).filter_by(name=stack_name)
-    return query.first()
+    return query.order_by(models.Stack.created_at).first()
 
 
 def stack_get(context, stack_id, show_deleted=False, eager_load=True):
@@ -635,7 +663,7 @@ def _get_sort_keys(sort_keys, mapping):
     :param mapping: a mapping from keys to DB column names
     :returns: filtered list of sort keys
     """
-    if isinstance(sort_keys, six.string_types):
+    if isinstance(sort_keys, str):
         sort_keys = [sort_keys]
     return [mapping[key] for key in sort_keys or [] if key in mapping]
 
@@ -765,12 +793,21 @@ def stack_count_all(context, filters=None,
 def stack_create(context, values):
     stack_ref = models.Stack()
     stack_ref.update(values)
+    stack_name = stack_ref.name
     stack_ref.save(context.session)
+
+    # Even though we just created a stack with this name, we may not find
+    # it again because some unit tests create stacks with deleted_at set. Also
+    # some backup stacks may not be found, for reasons that are unclear.
+    earliest = stack_get_by_name(context, stack_name)
+    if earliest is not None and earliest.id != stack_ref.id:
+        context.session.query(models.Stack).filter_by(id=stack_ref.id).delete()
+        raise exception.StackExists(stack_name=stack_name)
+
     return stack_ref
 
 
-@oslo_db_api.wrap_db_retry(max_retries=3, retry_on_deadlock=True,
-                           retry_interval=0.5, inc_retry_interval=True)
+@retry_on_db_error
 def stack_update(context, stack_id, values, exp_trvsl=None):
     session = context.session
     with session.begin(subtransactions=True):
@@ -828,7 +865,9 @@ def _is_duplicate_error(exc):
 
 
 @oslo_db_api.wrap_db_retry(max_retries=3, retry_on_deadlock=True,
-                           retry_interval=0.5, inc_retry_interval=True,
+                           retry_on_disconnect=True,
+                           retry_interval=0.5,
+                           inc_retry_interval=True,
                            exception_checker=_is_duplicate_error)
 def stack_lock_create(context, stack_id, engine_id):
     with db_context.writer.independent.using(context) as session:
@@ -914,7 +953,7 @@ def user_creds_create(context):
     else:
         user_creds_ref.update(values)
         method, password = crypt.encrypt(values['password'])
-        if len(six.text_type(password)) > 255:
+        if len(str(password)) > 255:
             raise exception.Error(_("Length of OS_PASSWORD after encryption"
                                     " exceeds Heat limit (255 chars)"))
         user_creds_ref.password = password
@@ -1126,8 +1165,7 @@ def _delete_event_rows(context, stack_id, limit):
     return retval
 
 
-@oslo_db_api.wrap_db_retry(max_retries=3, retry_on_deadlock=True,
-                           retry_interval=0.5, inc_retry_interval=True)
+@retry_on_db_error
 def event_create(context, values):
     if 'stack_id' in values and cfg.CONF.max_events_per_stack:
         # only count events and purge on average
@@ -1141,7 +1179,7 @@ def event_create(context, values):
                 _delete_event_rows(context, values['stack_id'],
                                    cfg.CONF.event_purge_batch_size)
             except db_exception.DBError as exc:
-                LOG.error('Failed to purge events: %s', six.text_type(exc))
+                LOG.error('Failed to purge events: %s', str(exc))
     event_ref = models.Event()
     event_ref.update(values)
     event_ref.save(context.session)
@@ -1235,7 +1273,7 @@ def software_deployment_update(context, deployment_id, values):
 def software_deployment_delete(context, deployment_id):
     deployment = software_deployment_get(context, deployment_id)
     session = context.session
-    with session.begin(subtransactions=True):
+    with session.begin():
         session.delete(deployment)
 
 
@@ -1539,8 +1577,7 @@ def sync_point_delete_all_by_stack_and_traversal(context, stack_id,
     return rows_deleted
 
 
-@oslo_db_api.wrap_db_retry(max_retries=3, retry_on_deadlock=True,
-                           retry_interval=0.5, inc_retry_interval=True)
+@retry_on_db_error
 def sync_point_create(context, values):
     values['entity_id'] = str(values['entity_id'])
     sync_point_ref = models.SyncPoint()
@@ -1556,8 +1593,7 @@ def sync_point_get(context, entity_id, traversal_id, is_update):
     )
 
 
-@oslo_db_api.wrap_db_retry(max_retries=3, retry_on_deadlock=True,
-                           retry_interval=0.5, inc_retry_interval=True)
+@retry_on_db_error
 def sync_point_update_input_data(context, entity_id,
                                  traversal_id, is_update, atomic_key,
                                  input_data):
@@ -1601,7 +1637,7 @@ def _db_encrypt_or_decrypt_template_params(
         batch_size=batch_size)
     next_batch = list(itertools.islice(template_batches, batch_size))
     while next_batch:
-        with session.begin(subtransactions=True):
+        with session.begin():
             for raw_template in next_batch:
                 try:
                     if verbose:
@@ -1632,7 +1668,7 @@ def _db_encrypt_or_decrypt_template_params(
                                     not param_schemata[param_name].hidden):
                                 continue
                             encrypted_val = crypt.encrypt(
-                                six.text_type(param_val), encryption_key)
+                                str(param_val), encryption_key)
                             env['parameters'][param_name] = encrypted_val
                             encrypted_params.append(param_name)
                             needs_update = True
@@ -1684,7 +1720,7 @@ def _db_encrypt_or_decrypt_resource_prop_data_legacy(
         batch_size=batch_size)
     next_batch = list(itertools.islice(resource_batches, batch_size))
     while next_batch:
-        with session.begin(subtransactions=True):
+        with session.begin():
             for resource in next_batch:
                 if not resource.properties_data:
                     continue
@@ -1731,7 +1767,7 @@ def _db_encrypt_or_decrypt_resource_prop_data(
         model=models.ResourcePropertiesData, batch_size=batch_size)
     next_batch = list(itertools.islice(rpd_batches, batch_size))
     while next_batch:
-        with session.begin(subtransactions=True):
+        with session.begin():
             for rpd in next_batch:
                 if not rpd.data:
                     continue
