@@ -15,7 +15,6 @@ import collections
 
 from oslo_log import log as logging
 from oslo_utils import timeutils
-import six
 
 from heat.common.i18n import _
 from heat.common import param_utils
@@ -60,12 +59,12 @@ def extract_args(params):
         kwargs[rpc_api.PARAM_ADOPT_STACK_DATA] = adopt_data
 
     tags = params.get(rpc_api.PARAM_TAGS)
-    if tags:
+    if tags is not None:
         if not isinstance(tags, list):
             raise ValueError(_('Invalid tags, not a list: %s') % tags)
 
         for tag in tags:
-            if not isinstance(tag, six.string_types):
+            if not isinstance(tag, str):
                 raise ValueError(_('Invalid tag, "%s" is not a string') % tag)
 
             if len(tag) > 80:
@@ -188,7 +187,7 @@ def format_stack_output(output_defn, resolve_value=True):
         except Exception as ex:
             # We don't need error raising, just adding output_error to
             # resulting dict.
-            result.update({rpc_api.OUTPUT_ERROR: six.text_type(ex)})
+            result.update({rpc_api.OUTPUT_ERROR: str(ex)})
         finally:
             result.update({rpc_api.OUTPUT_VALUE: value})
 
@@ -212,7 +211,7 @@ def format_stack(stack, preview=False, resolve_outputs=True):
         rpc_api.STACK_UPDATED_TIME: updated_time,
         rpc_api.STACK_DELETION_TIME: deleted_time,
         rpc_api.STACK_NOTIFICATION_TOPICS: [],  # TODO(therve) Not implemented
-        rpc_api.STACK_PARAMETERS: stack.parameters.map(six.text_type),
+        rpc_api.STACK_PARAMETERS: stack.parameters.map(str),
         rpc_api.STACK_DESCRIPTION: stack.t[stack.t.DESCRIPTION],
         rpc_api.STACK_TMPL_DESCRIPTION: stack.t[stack.t.DESCRIPTION],
         rpc_api.STACK_CAPABILITIES: [],   # TODO(?) Not implemented yet
@@ -221,7 +220,7 @@ def format_stack(stack, preview=False, resolve_outputs=True):
         rpc_api.STACK_OWNER: stack.username,
         rpc_api.STACK_PARENT: stack.owner_id,
         rpc_api.STACK_USER_PROJECT_ID: stack.stack_user_project_id,
-        rpc_api.STACK_TAGS: stack.tags,
+        rpc_api.STACK_TAGS: stack.tags or None,
     }
 
     if not preview:
