@@ -13,8 +13,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import six
-
 import eventlet.queue
 import functools
 
@@ -83,7 +81,7 @@ class CheckResource(object):
                                  'during resource %s' % rsrc.action)
                 rsrc.state_set(rsrc.action,
                                rsrc.FAILED,
-                               six.text_type(status_reason))
+                               str(status_reason))
                 return True
         elif (rs_obj.engine_id is None and
               rs_obj.current_template_id == prev_template_id):
@@ -105,11 +103,11 @@ class CheckResource(object):
 
     def _retrigger_new_traversal(self, cnxt, current_traversal, is_update,
                                  stack_id, rsrc_id):
-            latest_stack = parser.Stack.load(cnxt, stack_id=stack_id,
-                                             force_reload=True)
-            if current_traversal != latest_stack.current_traversal:
-                self.retrigger_check_resource(cnxt, is_update, rsrc_id,
-                                              latest_stack)
+        latest_stack = parser.Stack.load(cnxt, stack_id=stack_id,
+                                         force_reload=True)
+        if current_traversal != latest_stack.current_traversal:
+            self.retrigger_check_resource(cnxt, is_update, rsrc_id,
+                                          latest_stack)
 
     def _handle_stack_timeout(self, cnxt, stack):
         failure_reason = u'Timed out'
@@ -163,6 +161,8 @@ class CheckResource(object):
 
             return True
         except exception.UpdateInProgress:
+            LOG.debug('Waiting for existing update to unlock resource %s',
+                      rsrc.id)
             if self._stale_resource_needs_retry(cnxt, rsrc, prev_template_id):
                 rpc_data = sync_point.serialize_input_data(self.input_data)
                 self._rpc_client.check_resource(cnxt,
@@ -170,16 +170,18 @@ class CheckResource(object):
                                                 current_traversal,
                                                 rpc_data, is_update,
                                                 adopt_stack_data)
+            else:
+                rsrc.handle_preempt()
         except exception.ResourceFailure as ex:
             action = ex.action or rsrc.action
             reason = 'Resource %s failed: %s' % (action,
-                                                 six.text_type(ex))
+                                                 str(ex))
             self._handle_resource_failure(cnxt, is_update, rsrc.id,
                                           stack, reason)
         except scheduler.Timeout:
             self._handle_resource_failure(cnxt, is_update, rsrc.id,
                                           stack, u'Timed out')
-        except CancelOperation as ex:
+        except CancelOperation:
             # Stack is already marked FAILED, so we just need to retrigger
             # in case a new traversal has started and is waiting on us.
             self._retrigger_new_traversal(cnxt, current_traversal, is_update,
@@ -304,14 +306,6 @@ class CheckResource(object):
         stack.adopt_stack_data = adopt_stack_data
         stack.thread_group_mgr = self.thread_group_mgr
 
-        if is_update:
-            if (rsrc.replaced_by is not None and
-                    rsrc.current_template_id != tmpl.id):
-                LOG.debug('Resource %s with id %s already replaced by %s; '
-                          'not checking',
-                          rsrc.name, resource_id, rsrc.replaced_by)
-                return
-
         try:
             check_resource_done = self._do_check_resource(cnxt,
                                                           current_traversal,
@@ -327,7 +321,7 @@ class CheckResource(object):
                                                   rsrc, stack)
         except BaseException as exc:
             with excutils.save_and_reraise_exception():
-                msg = six.text_type(exc)
+                msg = str(exc)
                 LOG.exception("Unexpected exception in resource check.")
                 self._handle_resource_failure(cnxt, is_update, rsrc.id,
                                               stack, msg)
