@@ -245,7 +245,6 @@ class Port(neutron.NeutronResource):
             constraints=[
                 constraints.CustomConstraint('mac_addr')
             ],
-            update_allowed=True,
         ),
         ALLOWED_ADDRESS_PAIRS: properties.Schema(
             properties.Schema.LIST,
@@ -266,7 +265,7 @@ class Port(neutron.NeutronResource):
                         _('IP address to allow through this port.'),
                         required=True,
                         constraints=[
-                            constraints.CustomConstraint('net_cidr')
+                            constraints.CustomConstraint('ip_or_cidr')
                         ]
                     ),
                 },
@@ -284,7 +283,8 @@ class Port(neutron.NeutronResource):
               'the bindings extension.'),
             constraints=[
                 constraints.AllowedValues(['normal', 'direct', 'macvtap',
-                                           'direct-physical', 'baremetal']),
+                                           'direct-physical', 'baremetal',
+                                           'virtio-forwarder']),
             ],
             support_status=support.SupportStatus(version='2015.1'),
             update_allowed=True,
@@ -384,7 +384,7 @@ class Port(neutron.NeutronResource):
             _("The attributes of the network owning the port. (The full list "
               "of response parameters can be found in the `Openstack "
               "Networking service API reference "
-              "<https://developer.openstack.org/api-ref/network/>`_.) The "
+              "<https://docs.openstack.org/api-ref/network/>`_.) The "
               "following examples demonstrate some (not all) possible "
               "expressions. (Obtains the network, the MTU (Maximum "
               "transmission unit), the network tags and the l2_adjacency "
@@ -543,7 +543,6 @@ class Port(neutron.NeutronResource):
         result = super(Port, self).parse_live_resource_data(
             resource_properties, resource_data)
         result[self.QOS_POLICY] = resource_data.get('qos_policy_id')
-        result.pop(self.MAC_ADDRESS)
         fixed_ips = resource_data.get(self.FIXED_IPS) or []
         if fixed_ips:
             result.update({self.FIXED_IPS: []})
