@@ -39,7 +39,8 @@ cfg.CONF.import_opt('default_user_data_format', 'heat.common.config')
 LOG = logging.getLogger(__name__)
 
 NOVA_MICROVERSIONS = (MICROVERSION_TAGS, MICROVERSION_STR_NETWORK,
-                      MICROVERSION_NIC_TAGS) = ('2.26', '2.37', '2.42')
+                      MICROVERSION_NIC_TAGS, MICROVERSION_PERSONALITY_REMOVED
+                      ) = ('2.26', '2.37', '2.42', '2.57')
 
 
 class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
@@ -703,12 +704,20 @@ class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
         ACCESSIPV4: attributes.Schema(
             _('The manually assigned alternative public IPv4 address '
               'of the server.'),
-            type=attributes.Schema.STRING
+            type=attributes.Schema.STRING,
+            support_status=support.SupportStatus(
+                status=support.DEPRECATED,
+                version='14.0.0',
+                previous_status=support.SupportStatus(version='2015.1')),
         ),
         ACCESSIPV6: attributes.Schema(
             _('The manually assigned alternative public IPv6 address '
               'of the server.'),
-            type=attributes.Schema.STRING
+            type=attributes.Schema.STRING,
+            support_status=support.SupportStatus(
+                status=support.DEPRECATED,
+                version='14.0.0',
+                previous_status=support.SupportStatus(version='2015.1'))
         ),
         CONSOLE_URLS: attributes.Schema(
             _("URLs of server's consoles. "
@@ -1585,6 +1594,14 @@ class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
         # retrieve provider's absolute limits if it will be needed
         metadata = self.properties[self.METADATA]
         personality = self.properties[self.PERSONALITY]
+
+        if personality:
+            if self.client_plugin().is_version_supported(
+                    MICROVERSION_PERSONALITY_REMOVED):
+                msg = (_('Cannot use the personality parameter as nova no '
+                         'longer supports it. Use user_data instead.'))
+                raise exception.StackValidationFailed(message=msg)
+
         if metadata or personality:
             limits = self.client_plugin().absolute_limits()
 
@@ -1748,7 +1765,11 @@ class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
     def check_snapshot_complete(self, image_id):
         image = self.client_plugin('glance').get_image(image_id)
         if image.status.lower() == self.IMAGE_ACTIVE:
-            return True
+            server = self.client_plugin().get_server(self.resource_id)
+            task_state = getattr(server, 'OS-EXT-STS:task_state', '')
+            if task_state not in {'image_uploading', 'image_snapshot_pending',
+                                  'image_snapshot', 'image_pending_upload'}:
+                return True
         elif image.status.lower() in (self.IMAGE_ERROR, self.IMAGE_DELETED):
             raise exception.Error(image.status)
 
