@@ -27,21 +27,23 @@ from heat.common import profiler
 from heat import version as hversion
 
 
+CONF = cfg.CONF
+
+
 def init_application():
     i18n.enable_lazy()
 
-    LOG = logging.getLogger('heat.api')
-
-    logging.register_options(cfg.CONF)
+    logging.register_options(CONF)
     version = hversion.version_info.version_string()
-    cfg.CONF(project='heat', prog='heat-api', version=version)
-    logging.setup(cfg.CONF, 'heat-api')
+    CONF(project='heat', prog='heat-api', version=version)
+    logging.setup(CONF, CONF.prog)
+    LOG = logging.getLogger(CONF.prog)
     config.set_config_defaults()
     messaging.setup()
 
-    port = cfg.CONF.heat_api.bind_port
-    host = cfg.CONF.heat_api.bind_host
-    profiler.setup('heat-api', host)
+    port = CONF.heat_api.bind_port
+    host = CONF.heat_api.bind_host
+    profiler.setup(CONF.prog, host)
     LOG.info('Starting Heat REST API on %(host)s:%(port)s',
              {'host': host, 'port': port})
     return config.load_paste_app()
