@@ -86,6 +86,7 @@ class KeystoneClientTest(common.HeatTestCase):
         self.m_client.assert_called_once_with(
             session=utils.AnyInstance(ks_session.Session),
             auth=self.mock_ks_auth,
+            connect_retries=2,
             region_name=None)
 
     def _stubs_auth(self, method='token', trust_scoped=True,
@@ -167,6 +168,7 @@ class KeystoneClientTest(common.HeatTestCase):
         if self.client:
             self.m_client.assert_any_call(
                 session=utils.AnyInstance(ks_session.Session),
+                connect_retries=2,
                 region_name=None)
         if self.stub_admin_auth:
             self.mock_admin_ks_auth.get_user_id.assert_called_once_with(
@@ -251,6 +253,9 @@ class KeystoneClientTest(common.HeatTestCase):
         ctx = utils.dummy_context()
         self.patchobject(ctx, '_create_auth_plugin')
         ctx.trust_id = None
+        user_options = dict(ignore_password_expiry=True,
+                            ignore_change_password_upon_first_use=True,
+                            ignore_lockout_failure_attempts=True)
 
         # mock keystone client functions
         self._stub_domain_admin_client()
@@ -266,7 +271,8 @@ class KeystoneClientTest(common.HeatTestCase):
             name='duser',
             password=None,
             default_project='aproject',
-            domain='adomain123')
+            domain='adomain123',
+            options=user_options)
         self.mock_ks_v3_client.roles.grant.assert_called_once_with(
             project='aproject',
             role='4546',
@@ -539,7 +545,17 @@ class KeystoneClientTest(common.HeatTestCase):
     def test_create_trust_context_trust_create_deletegate_all_roles(self):
         self._test_create_trust_context_trust_create()
 
-    def _test_create_trust_context_trust_create(self, delegate_roles=None):
+    def test_create_trust_context_trust_create_with_enabled_redelegation(self):
+        cfg.CONF.set_override('reauthentication_auth_method', 'trusts')
+        cfg.CONF.set_override('allow_trusts_redelegation', True)
+        self._test_create_trust_context_trust_create(redelegate=True)
+
+    def test_create_trust_context_trust_create_with_no_redelegation(self):
+        cfg.CONF.set_override('reauthentication_auth_method', 'trusts')
+        self._test_create_trust_context_trust_create()
+
+    def _test_create_trust_context_trust_create(self, delegate_roles=None,
+                                                redelegate=False):
 
         """Test create_trust_context when creating a trust."""
 
@@ -571,6 +587,7 @@ class KeystoneClientTest(common.HeatTestCase):
         self.m_load_auth.assert_called_once_with(
             cfg.CONF, 'trustee', trust_id=None)
         self.mock_ks_v3_client.trusts.create.assert_called_once_with(
+            allow_redelegation=redelegate,
             trustor_user='5678',
             trustee_user='1234',
             project='42',
@@ -630,6 +647,7 @@ class KeystoneClientTest(common.HeatTestCase):
         self.m_load_auth.assert_called_with(
             cfg.CONF, 'trustee', trust_id=None)
         self.mock_ks_v3_client.trusts.create.assert_called_once_with(
+            allow_redelegation=False,
             trustor_user='5678',
             trustee_user='1234',
             project='42',
@@ -1433,6 +1451,52 @@ class KeystoneClientTest(common.HeatTestCase):
         self.assertIsNone(heat_ks_client.delete_stack_domain_project(
             project_id='aprojectid'))
 
+    def test_server_keystone_endpoint_url_config(self):
+        """Return non fallback url path."""
+        cfg.CONF.set_override('server_keystone_endpoint_type', 'public')
+        ctx = utils.dummy_context()
+        ctx.trust_id = None
+        heat_ks_client = heat_keystoneclient.KeystoneClient(ctx)
+        fallback_url = 'http://server.fallback.test:5000/v3'
+        auth_ref = heat_ks_client.context.auth_plugin.get_access(
+            heat_ks_client.session)
+        auth_ref.service_catalog.get_urls = mock.MagicMock()
+        auth_ref.service_catalog.get_urls.return_value = [
+            'http://server.public.test:5000']
+        self.assertEqual(
+            heat_ks_client.server_keystone_endpoint_url(fallback_url),
+            'http://server.public.test:5000/v3')
+        cfg.CONF.clear_override('server_keystone_endpoint_type')
+
+    def test_server_keystone_endpoint_url_no_config(self):
+        """Return fallback as no config option specified."""
+        ctx = utils.dummy_context()
+        ctx.trust_id = None
+        heat_ks_client = heat_keystoneclient.KeystoneClient(ctx)
+        cfg.CONF.clear_override('server_keystone_endpoint_type')
+        fallback_url = 'http://server.fallback.test:5000/v3'
+        self.assertEqual(heat_ks_client.server_keystone_endpoint_url(
+            fallback_url), fallback_url)
+
+    def test_server_keystone_endpoint_url_auth_exception(self):
+        """Authorization call fails, return fallback."""
+        cfg.CONF.set_override('server_keystone_endpoint_type', 'public')
+        ctx = utils.dummy_context()
+        ctx.trust_id = None
+        heat_ks_client = heat_keystoneclient.KeystoneClient(ctx)
+        auth_ref = heat_ks_client.context.auth_plugin.get_access(
+            heat_ks_client.session)
+        auth_ref.service_catalog.get_urls = mock.MagicMock()
+        auth_ref.service_catalog.get_urls.return_value = [
+            'http://server.public.test:5000']
+        heat_ks_client.context.auth_plugin.get_access = mock.MagicMock()
+        heat_ks_client.context.auth_plugin.get_access.side_effect = (
+            kc_exception.Unauthorized)
+        fallback_url = 'http://server.fallback.test:5000/v3'
+        self.assertEqual(heat_ks_client.server_keystone_endpoint_url(
+            fallback_url), fallback_url)
+        cfg.CONF.clear_override('server_keystone_endpoint_type')
+
 
 class KeystoneClientTestDomainName(KeystoneClientTest):
     def setUp(self):
@@ -1456,6 +1520,7 @@ class KeystoneClientTestDomainName(KeystoneClientTest):
         self.m_client.assert_called_once_with(
             session=utils.AnyInstance(ks_session.Session),
             auth=self.mock_ks_auth,
+            connect_retries=2,
             region_name=None)
 
     def _stub_domain_admin_client(self, domain_id='adomain123'):
